@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Nominum, Inc.
+# Copyright (C) 2003-2007 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -68,8 +68,11 @@ class Answer(object):
     Instances of this class bundle up the result of a successful DNS
     resolution.
 
-    For convenience, the answer is iterable.  "for a in answer" is
-    equivalent to "for a in answer.rrset".
+    For convenience, the answer object implements much of the sequence
+    protocol, forwarding to its rrset.  E.g. "for a in answer" is
+    equivalent to "for a in answer.rrset", "answer[i]" is equivalent
+    to "answer.rrset[i]", and "answer[i:j]" is equivalent to
+    "answer.rrset[i:j]".
 
     Note that CNAMEs or DNAMEs in the response may mean that answer
     node's name might not be the query name.
@@ -142,6 +145,18 @@ class Answer(object):
     def __iter__(self):
         return iter(self.rrset)
 
+    def __getitem__(self, i):
+        return self.rrset[i]
+
+    def __delitem__(self, i):
+        del self.rrset[i]
+
+    def __getslice__(self, i, j):
+        return self.rrset[i:j]
+
+    def __delslice__(self, i, j):
+        del self.rrset[i:j]
+
 class Cache(object):
     """Simple DNS answer cache.
 
@@ -224,7 +239,7 @@ class Cache(object):
         else:
             self.data = {}
             self.next_cleaning = time.time() + self.cleaning_interval
-    
+
 class Resolver(object):
     """DNS stub resolver
 
@@ -302,7 +317,13 @@ class Resolver(object):
         a string, it is used as the name of the file to open; otherwise it
         is treated as the file itself."""
         if isinstance(f, str) or isinstance(f, unicode):
-            f = open(f, 'r')
+            try:
+                f = open(f, 'r')
+            except IOError:
+                # /etc/resolv.conf doesn't exist, can't be read, etc.
+                # We'll just use the default resolver configuration.
+                self.nameservers = ['127.0.0.1']
+                return
             want_close = True
         else:
             want_close = False
@@ -326,10 +347,27 @@ class Resolver(object):
         if len(self.nameservers) == 0:
             self.nameservers.append('127.0.0.1')
 
-    def _config_win32_nameservers(self, nameservers, split_char=','):
+    def _determine_split_char(self, entry):
+        #
+        # The windows registry irritatingly changes the list element
+        # delimiter in between ' ' and ',' (and vice-versa) in various
+        # versions of windows.
+        #
+        if entry.find(' ') >= 0:
+            split_char = ' '
+        elif entry.find(',') >= 0:
+            split_char = ','
+        else:
+            # probably a singleton; treat as a space-separated list.
+            split_char = ' '
+        return split_char
+
+    def _config_win32_nameservers(self, nameservers):
         """Configure a NameServer registry entry."""
         # we call str() on nameservers to convert it from unicode to ascii
-        ns_list = str(nameservers).split(split_char)
+        nameservers = str(nameservers)
+        split_char = self._determine_split_char(nameservers)
+        ns_list = nameservers.split(split_char)
         for ns in ns_list:
             if not ns in self.nameservers:
                 self.nameservers.append(ns)
@@ -342,7 +380,9 @@ class Resolver(object):
     def _config_win32_search(self, search):
         """Configure a Search registry entry."""
         # we call str() on search to convert it from unicode to ascii
-        search_list = str(search).split(',')
+        search = str(search)
+        split_char = self._determine_split_char(search)
+        search_list = search.split(split_char)
         for s in search_list:
             if not s in self.search:
                 self.search.append(dns.name.from_text(s))
@@ -367,9 +407,7 @@ class Resolver(object):
             except WindowsError:
                 servers = None
             if servers:
-                # Annoyingly, the DhcpNameServer list is apparently space
-                # separated instead of comma separated like NameServer.
-                self._config_win32_nameservers(servers, ' ')
+                self._config_win32_nameservers(servers)
                 try:
                     dom, rtype = _winreg.QueryValueEx(key, 'DhcpDomain')
                     if dom:
@@ -381,7 +419,7 @@ class Resolver(object):
         except WindowsError:
             search = None
         if search:
-            self._config_win32_search(servers)
+            self._config_win32_search(search)
 
     def read_registry(self):
         """Extract resolver configuration from the Windows registry."""
@@ -415,16 +453,10 @@ class Resolver(object):
                             guid = _winreg.EnumKey(interfaces, i)
                             i += 1
                             key = _winreg.OpenKey(interfaces, guid)
+                            if not self._win32_is_nic_enabled(lm, guid, key):
+                                continue
                             try:
-                                # enabled interfaces seem to have a non-empty
-                                # NTEContextList
-                                try:
-                                    (nte, ttype) = _winreg.QueryValueEx(key,
-                                                             'NTEContextList')
-                                except WindowsError:
-                                    nte = None
-                                if nte:
-                                    self._config_win32_fromkey(key)
+                                self._config_win32_fromkey(key)
                             finally:
                                 key.Close()
                         except EnvironmentError:
@@ -434,6 +466,61 @@ class Resolver(object):
         finally:
             lm.Close()
 
+    def _win32_is_nic_enabled(self, lm, guid, interface_key):
+         # Look in the Windows Registry to determine whether the network
+         # interface corresponding to the given guid is enabled.
+         #
+         # (Code contributed by Paul Marks, thanks!)
+         #
+         try:
+             # This hard-coded location seems to be consistent, at least
+             # from Windows 2000 through Vista.
+             connection_key = _winreg.OpenKey(
+                 lm,
+                 r'SYSTEM\CurrentControlSet\Control\Network'
+                 r'\{4D36E972-E325-11CE-BFC1-08002BE10318}'
+                 r'\%s\Connection' % guid)
+
+             try:
+                 # The PnpInstanceID points to a key inside Enum
+                 (pnp_id, ttype) = _winreg.QueryValueEx(
+                     connection_key, 'PnpInstanceID')
+
+                 if ttype != _winreg.REG_SZ:
+                     raise ValueError
+
+                 device_key = _winreg.OpenKey(
+                     lm, r'SYSTEM\CurrentControlSet\Enum\%s' % pnp_id)
+
+                 try:
+                     # Get ConfigFlags for this device
+                     (flags, ttype) = _winreg.QueryValueEx(
+                         device_key, 'ConfigFlags')
+
+                     if ttype != _winreg.REG_DWORD:
+                         raise ValueError
+
+                     # Based on experimentation, bit 0x1 indicates that the
+                     # device is disabled.
+                     return not (flags & 0x1)
+
+                 finally:
+                     device_key.Close()
+             finally:
+                 connection_key.Close()
+         except (EnvironmentError, ValueError):
+             # Pre-vista, enabled interfaces seem to have a non-empty
+             # NTEContextList; this was how dnspython detected enabled
+             # nics before the code above was contributed.  We've retained
+             # the old method since we don't know if the code above works
+             # on Windows 95/98/ME.
+             try:
+                 (nte, ttype) = _winreg.QueryValueEx(interface_key,
+                                                     'NTEContextList')
+                 return nte is not None
+             except WindowsError:
+                 return False
+
     def _compute_timeout(self, start):
         now = time.time()
         if now < start:
@@ -474,7 +561,7 @@ class Resolver(object):
         @raises NoNameservers: no non-broken nameservers are available to
         answer the question."""
         
-        if isinstance(qname, str):
+        if isinstance(qname, (str, unicode)):
             qname = dns.name.from_text(qname, None)
         if isinstance(rdtype, str):
             rdtype = dns.rdatatype.from_text(rdtype)
@@ -511,7 +598,7 @@ class Resolver(object):
             while response is None:
                 if len(nameservers) == 0:
                     raise NoNameservers
-                for nameserver in nameservers:
+                for nameserver in nameservers[:]:
                     timeout = self._compute_timeout(start)
                     try:
                         if tcp:
@@ -554,6 +641,8 @@ class Resolver(object):
                     if rcode != dns.rcode.SERVFAIL:
                         nameservers.remove(nameserver)
                     response = None
+                if not response is None:
+                    break
                 #
                 # All nameservers failed!
                 #
@@ -642,7 +731,7 @@ def zone_for_name(name, rdclass=dns.rdataclass.IN, tcp=False, resolver=None):
     @type resolver: dns.resolver.Resolver object or None
     @rtype: dns.name.Name"""
 
-    if isinstance(name, str):
+    if isinstance(name, (str, unicode)):
         name = dns.name.from_text(name, dns.name.root)
     if resolver is None:
         resolver = get_default_resolver()
