@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Nominum, Inc.
+# Copyright (C) 2003-2007 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -25,6 +25,7 @@ import dns.node
 import dns.rdataclass
 import dns.rdatatype
 import dns.rdata
+import dns.rrset
 import dns.tokenizer
 import dns.ttl
 
@@ -40,6 +41,10 @@ class NoNS(BadZone):
     """The zone has no NS RRset at its origin."""
     pass
 
+class UnknownOrigin(BadZone):
+    """The zone's origin is unknown."""
+    pass
+
 class Zone(object):
     """A DNS zone.
 
@@ -102,7 +107,7 @@ class Zone(object):
         return not self.__eq__(other)
 
     def _validate_name(self, name):
-        if isinstance(name, str):
+        if isinstance(name, (str, unicode)):
             name = dns.name.from_text(name, None)
         elif not isinstance(name, dns.name.Name):
             raise KeyError, \
@@ -163,7 +168,7 @@ class Zone(object):
         @param create: should the node be created if it doesn't exist?
         @type create: bool
         @raises KeyError: the name is not known and create was not specified.
-        @rtype dns.node.Node object
+        @rtype: dns.node.Node object
         """
         
         name = self._validate_name(name)
@@ -186,7 +191,7 @@ class Zone(object):
         @type name: dns.name.Name object or string
         @param create: should the node be created if it doesn't exist?
         @type create: bool
-        @rtype dns.node.Node object or None
+        @rtype: dns.node.Node object or None
         """
 
         try:
@@ -539,11 +544,14 @@ class _MasterReader(object):
     (None if no $INCLUDE is active).
     @ivar allow_include: is $INCLUDE allowed?
     @type allow_include: bool
+    @ivar check_origin: should sanity checks of the origin node be done?
+    The default is True.
+    @type check_origin: bool
     """
 
     def __init__(self, tok, origin, rdclass, relativize, zone_factory=Zone,
-                 allow_include=False):
-        if isinstance(origin, str):
+                 allow_include=False, check_origin=True):
+        if isinstance(origin, (str, unicode)):
             origin = dns.name.from_text(origin)
         self.tok = tok
         self.current_origin = origin
@@ -554,6 +562,7 @@ class _MasterReader(object):
         self.saved_state = []
         self.current_file = None
         self.allow_include = allow_include
+        self.check_origin = check_origin
 
     def _eat_line(self):
         while 1:
@@ -564,6 +573,8 @@ class _MasterReader(object):
     def _rr_line(self):
         """Process one line from a DNS master file."""
         # Name
+        if self.current_origin is None:
+            raise UnknownOrigin
         token = self.tok.get(want_leading = True)
         if token[0] != dns.tokenizer.WHITESPACE:
             self.last_name = dns.name.from_text(token[1], self.current_origin)
@@ -673,6 +684,8 @@ class _MasterReader(object):
                     elif u == '$ORIGIN':
                         self.current_origin = self.tok.get_name()
                         self.tok.get_eol()
+                        if self.zone.origin is None:
+                            self.zone.origin = self.current_origin
                     elif u == '$INCLUDE' and self.allow_include:
                         token = self.tok.get()
                         if token[0] != dns.tokenizer.QUOTED_STRING:
@@ -713,15 +726,19 @@ class _MasterReader(object):
                   "%s:%d: %s" % (filename, line_number, detail)
         
         # Now that we're done reading, do some basic checking of the zone.
-        self.zone.check_origin()
+        if self.check_origin:
+            self.zone.check_origin()
 
-def from_text(text, origin, rdclass = dns.rdataclass.IN, relativize = True,
-              zone_factory=Zone, filename=None, allow_include=False):
+def from_text(text, origin = None, rdclass = dns.rdataclass.IN,
+              relativize = True, zone_factory=Zone, filename=None,
+              allow_include=False, check_origin=True):
     """Build a zone object from a master file format string.
 
     @param text: the master file format input
     @type text: string.
-    @param origin: The origin of the zone.
+    @param origin: The origin of the zone; if not specified, the first
+    $ORIGIN statement in the master file will determine the origin of the
+    zone.
     @type origin: dns.name.Name object or string
     @param rdclass: The zone's rdata class; the default is class IN.
     @type rdclass: int
@@ -731,9 +748,12 @@ def from_text(text, origin, rdclass = dns.rdataclass.IN, relativize = True,
     @type zone_factory: function returning a Zone
     @param filename: The filename to emit when describing where an error
     occurred; the default is '<string>'.
+    @type filename: string
     @param allow_include: is $INCLUDE allowed?
     @type allow_include: bool
-    @type filename: string
+    @param check_origin: should sanity checks of the origin node be done?
+    The default is True.
+    @type check_origin: bool
     @raises dns.zone.NoSOA: No SOA RR was found at the zone origin
     @raises dns.zone.NoNS: No NS RRset was found at the zone origin
     @rtype: dns.zone.Zone object
@@ -747,17 +767,21 @@ def from_text(text, origin, rdclass = dns.rdataclass.IN, relativize = True,
         filename = '<string>'
     tok = dns.tokenizer.Tokenizer(text, filename)
     reader = _MasterReader(tok, origin, rdclass, relativize, zone_factory,
-                           allow_include=allow_include)
+                           allow_include=allow_include,
+                           check_origin=check_origin)
     reader.read()
     return reader.zone
 
-def from_file(f, origin, rdclass = dns.rdataclass.IN, relativize = True,
-              zone_factory=Zone, filename=None, allow_include=True):
+def from_file(f, origin = None, rdclass = dns.rdataclass.IN,
+              relativize = True, zone_factory=Zone, filename=None,
+              allow_include=True, check_origin=True):
     """Read a master file and build a zone object.
 
     @param f: file or string.  If I{f} is a string, it is treated
     as the name of a file to open.
-    @param origin: The origin of the zone.
+    @param origin: The origin of the zone; if not specified, the first
+    $ORIGIN statement in the master file will determine the origin of the
+    zone.
     @type origin: dns.name.Name object or string
     @param rdclass: The zone's rdata class; the default is class IN.
     @type rdclass: int
@@ -771,6 +795,9 @@ def from_file(f, origin, rdclass = dns.rdataclass.IN, relativize = True,
     @type filename: string
     @param allow_include: is $INCLUDE allowed?
     @type allow_include: bool
+    @param check_origin: should sanity checks of the origin node be done?
+    The default is True.
+    @type check_origin: bool
     @raises dns.zone.NoSOA: No SOA RR was found at the zone origin
     @raises dns.zone.NoNS: No NS RRset was found at the zone origin
     @rtype: dns.zone.Zone object
@@ -795,7 +822,7 @@ def from_file(f, origin, rdclass = dns.rdataclass.IN, relativize = True,
         
     try:
         z = from_text(f, origin, rdclass, relativize, zone_factory,
-                      filename, allow_include)
+                      filename, allow_include, check_origin)
     finally:
         if want_close:
             f.close()
